@@ -19,8 +19,6 @@ no Test::Stream::Exporter;
 use Carp qw/croak/;
 use Scalar::Util qw/reftype blessed/;
 
-use Test::Stream::Block;
-
 use Test::Stream::Compare qw/-all/;
 use Test::Stream::Context qw/context/;
 use Test::Stream::Util qw/rtype/;
@@ -283,10 +281,7 @@ sub _build_set {
     my $set;
     if ($btype eq 'CODE') {
         $set = build('Test::Stream::Compare::Set', $builder);
-        my @caller = caller(1);
-        my $block = Test::Stream::Block->new(coderef => $builder, caller => \@caller);
-        $set->set_file($block->file);
-        $set->set_lines([$block->start_line, $block->end_line]);
+        $set->set_builder($builder);
     }
     else {
         $set = Test::Stream::Compare::Set->new(checks => [@_]);
@@ -324,10 +319,8 @@ sub event($;$) {
     }
     elsif (reftype($spec) eq 'CODE') {
         $event = build('Test::Stream::Compare::Event', $spec);
-        my $block = Test::Stream::Block->new(coderef => $spec, caller => \@caller);
-        $event->set_file($block->file);
-        $event->set_lines([$block->start_line, $block->end_line]);
         $event->set_etype($intype),
+        $event->set_builder($spec);
     }
     else {
         my $refcheck = Test::Stream::Compare::Hash->new(
@@ -357,8 +350,9 @@ sub convert {
     if ($thing && blessed($thing) && $thing->isa('Test::Stream::Compare')) {
         return $thing unless $thing->isa('Test::Stream::Compare::Wildcard');
         my $newthing = convert($thing->expect, $strict);
-        $newthing->set_file($thing->file)   unless $newthing->file;
-        $newthing->set_lines($thing->lines) unless $newthing->lines;
+        $newthing->set_builder($thing->builder) unless $newthing->builder;
+        $newthing->set_file($thing->_file)      unless $newthing->_file;
+        $newthing->set_lines($thing->_lines)    unless $newthing->_lines;
         return $newthing;
     }
 
